/**
 * @module package/quiqqer/employee/bin/backend/controls/employee/AddressEdit
 * @author www.pcsg.de (Henning Leutz)
 */
define('package/quiqqer/employee/bin/backend/controls/employee/AddressEdit', [

    'qui/QUI',
    'qui/controls/Control',
    'package/quiqqer/countries/bin/Countries',
    'Users',
    'Locale',
    'Ajax',
    'Mustache',

    'text!package/quiqqer/employee/bin/backend/controls/employee/AddressEdit.html'

], function(QUI, QUIControl, Countries, Users, QUILocale, QUIAjax, Mustache, template) {
    'use strict';

    var lg = 'quiqqer/employee';

    return new Class({

        Extends: QUIControl,
        Type: 'package/quiqqer/employee/bin/backend/controls/employee/AddressEdit',

        Binds: [
            '$onInject'
        ],

        options: {
            addressId: false
        },

        initialize: function(options) {
            this.parent(options);

            this.$data = {};

            this.addEvents({
                onInject: this.$onInject
            });
        },

        /**
         * create the DOMNode element
         *
         * @return {HTMLDivElement}
         */
        create: function() {
            this.$Elm = this.parent();
            this.$Elm.addClass('quiqqer-employee-address-edit');

            this.$Elm.set('html', Mustache.render(template, {
                titleAddress: QUILocale.get('quiqqer/core', 'address'),
                textAddressCompany: QUILocale.get('quiqqer/core', 'company'),
                textAddressSalutation: QUILocale.get('quiqqer/core', 'salutation'),
                textAddressFirstname: QUILocale.get('quiqqer/core', 'firstname'),
                textAddressLastname: QUILocale.get('quiqqer/core', 'lastname'),
                textAddressStreet: QUILocale.get('quiqqer/core', 'street'),
                textAddressZIP: QUILocale.get('quiqqer/core', 'zip'),
                textAddressCity: QUILocale.get('quiqqer/core', 'city'),
                textAddressCountry: QUILocale.get('quiqqer/core', 'country')
            }));

            return this.$Elm;
        },

        /**
         * event : on inject
         */
        $onInject: function() {
            var self = this;

            Countries.getCountries().then(function(countries) {
                for (var code in countries) {
                    if (!countries.hasOwnProperty(code)) {
                        continue;
                    }

                    new Element('option', {
                        html: countries[code],
                        value: code
                    }).inject(self.getElm().getElement('[name="address-country"]'));
                }

                // fetch address
                QUIAjax.get('ajax_users_address_get', function(result) {
                    var Form = self.getElm().getElement('form'),
                        elements = Form.elements;

                    self.$data = result;

                    elements['address-salutation'].value = result.salutation;
                    elements['address-company'].value = result.company;
                    elements['address-firstname'].value = result.firstname;
                    elements['address-lastname'].value = result.lastname;
                    elements['address-street_no'].value = result.street_no;
                    elements['address-zip'].value = result.zip;
                    elements['address-city'].value = result.city;
                    elements['address-country'].value = result.country;

                    self.fireEvent('load', [self]);
                }, {
                    aid: self.getAttribute('addressId')
                });
            });
        },

        /**
         *
         * @return {string}
         */
        getAddressId: function() {
            if (!this.getAttribute('addressId')) {
                return '';
            }

            return this.getAttribute('addressId');
        },

        /**
         * Return the firstname of the address
         *
         * @return {string}
         */
        getFirstname: function() {
            if (typeof this.$data.firstname === 'undefined') {
                return '';
            }

            return this.$data.firstname;
        },

        /**
         * Return the lastname of the address
         *
         * @return {string}
         */
        getLastname: function() {
            if (typeof this.$data.lastname === 'undefined') {
                return '';
            }

            return this.$data.lastname;
        },

        /**
         * alias for save
         *
         * @return {*|Promise}
         */
        save: function() {
            return this.update();
        },

        /**
         * Updates the data to the address
         *
         * @return {Promise}
         */
        update: function() {
            var self = this;

            return new Promise(function(resolve, reject) {
                var Form = self.getElm().getElement('form'),
                    elements = Form.elements;

                QUIAjax.post('ajax_users_address_save', resolve, {
                    'package': 'quiqqer/employee',
                    onError: reject,
                    aid: self.getAttribute('addressId'),
                    data: JSON.encode({
                        company: elements['address-company'].value,
                        salutation: elements['address-salutation'].value,
                        firstname: elements['address-firstname'].value,
                        lastname: elements['address-lastname'].value,
                        street_no: elements['address-street_no'].value,
                        zip: elements['address-zip'].value,
                        city: elements['address-city'].value,
                        country: elements['address-country'].value
                    })
                });
            });
        }
    });
});
